﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace PWTEditor.SaveFile
{
    public class SaveBlock : ChecksumBlock
    {
        public override byte[] Data
        {
            get
            {
                return base.Data;
            }
            set
            {
                base.Data = value;
                Size = (uint)(DataSize + footerSize);
                Checksum = CalcChecksum();
            }
        }

        public override int DataSize { get { return Data.Length; } }
        protected static readonly int footerSize = 0x10;

        public virtual uint SaveCount { get; set; }
        public virtual uint Size { get; protected set; }
        protected virtual uint Magic { get; set; }
        protected virtual ushort Unknown { get; set; }

        protected static readonly uint saveBlockMagic = 0x31053527;

        public SaveBlock() : this(new byte[0]) { }
        public SaveBlock(byte[] data)
        {
            Data = data;
            SaveCount = 0;
            Size = (uint)(footerSize + data.Length);
            Magic = saveBlockMagic;
            Unknown = 0;
        }
        public SaveBlock(Stream input, int blockSize)
        {
            // load save block
            LoadBlock(input, blockSize, footerSize);
            
            // check if save block is valid
            CheckMagic();
            CheckSize();
            CheckChecksum();
        }

        protected override void LoadFooter(Stream input)
        {
            BinaryReader reader = new BinaryReader(input);

            SaveCount = reader.ReadUInt32();
            Size = reader.ReadUInt32();
            Magic = reader.ReadUInt32();
            Unknown = reader.ReadUInt16();
            Checksum = reader.ReadUInt16();
        }
        protected virtual void CheckMagic()
        {
            if (Magic != saveBlockMagic)
                throw new InvalidDataException("Magic number 0x" + Magic.ToString("X8") + " is wrong (expected: 0x" + saveBlockMagic.ToString("X8") + ").");
        }
        protected virtual void CheckSize()
        {
            int blockSize = Data.Length + footerSize;
            if (blockSize != Size)
                throw new InvalidDataException("Size " + blockSize + " is incorrect (expected: " + Size + " bytes).");
        }

        protected override void WriteFooter(Stream output)
        {
            BinaryWriter writer = new BinaryWriter(output);

            writer.Write(SaveCount);
            writer.Write(Size);
            writer.Write(Magic);
            writer.Write(Unknown);
            writer.Write(Checksum);
        }
    }
}
