﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace PWTEditor.SaveFile.Tournaments
{
    public class TournamentChecksumBlock : ChecksumBlock
    {
        protected static readonly int blockSize = 0x1304;
        protected static readonly int footerSize = 0xF0;
        protected static readonly int padding1Size = 0xE8;
        protected static readonly int padding2Size = 0x2;

        public override ushort Checksum
        {
            get
            {
                return base.Checksum;
            }
            protected set
            {
                base.Checksum = value;
                checksum1 = value;
                checksum2 = value;
            }
        }

        protected ushort checksum1 { get; set; }
        protected ushort checksum2 { get; set; }

        public ushort SaveCount { get; set; }
        public byte[] Padding1 { get; set; }
        public byte[] Padding2 { get; set; }

        public TournamentChecksumBlock() : this(new byte[0], 0) { }
        public TournamentChecksumBlock(TournamentData td) : this(td.ToData().ToArray(), 0) { }
        public TournamentChecksumBlock(byte[] data) : this(data, 0) { }
        public TournamentChecksumBlock(byte[] data, int offset)
        {
            byte[] buffer = new byte[blockSize - footerSize];
            Array.Copy(data, offset, buffer, 0, Math.Min(data.Length, buffer.Length));

            Data = data;
            SaveCount = 1;
            Padding1 = new byte[padding1Size];
            Padding2 = new byte[padding2Size];
        }
        public TournamentChecksumBlock(Stream input)
        {
            LoadBlock(input, blockSize, footerSize);

            CheckSameChecksum();
            CheckChecksum();
        }

        protected override void LoadFooter(Stream input)
        {
            BinaryReader reader = new BinaryReader(input);

            SaveCount = reader.ReadUInt16();
            Checksum = checksum1 = reader.ReadUInt16();
            Padding1 = reader.ReadBytes(padding1Size);
            checksum2 = reader.ReadUInt16();
            Padding2 = reader.ReadBytes(padding2Size);
        }
        protected void CheckSameChecksum()
        {
            if (checksum1 != checksum2)
                throw new InvalidDataException("Checksums 0x" + checksum1.ToString("X4") + " and 0x" + checksum2.ToString("X4") + " are not equal.");
        }

        protected override void WriteFooter(Stream output)
        {
            BinaryWriter writer = new BinaryWriter(output);

            writer.Write(SaveCount);
            writer.Write(checksum1);
            writer.Write(Padding1);
            writer.Write(checksum2);
            writer.Write(Padding2);
        }

        public TournamentData TournamentData
        {
            get
            {
                return new TournamentData(new MemoryStream(Data));
            }
            set
            {
                Data =value.ToData().ToArray();
            }
        }
    }
}
