﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using PWTEditor.Tournaments;

namespace PWTEditor.SaveFile.Tournaments
{
    public class TournamentData : ChecksumBlock
    {
        protected readonly static int blockSize = 0x1214;
        protected readonly static int footerSize = 0x4;

        protected byte[] Padding { get; set; }

        public TournamentData() : this(new byte[0], 0) { }
        public TournamentData(Tournament tournament) : this(tournament.Data, 0) { }
        public TournamentData(byte[] data) : this(data, 0) { }
        public TournamentData(byte[] data, int offset)
        {
            byte[] buffer = new byte[blockSize - footerSize];
            Array.Copy(data, offset, buffer, 0, Math.Min(data.Length, buffer.Length));

            Data = buffer;
            Padding = new byte[2];
        }
        public TournamentData(Stream input)
        {
            LoadBlock(input, blockSize, footerSize);
            CheckChecksum();
        }

        protected override void LoadFooter(Stream input)
        {
            BinaryReader reader = new BinaryReader(input);

            Checksum = reader.ReadUInt16();
            Padding = reader.ReadBytes(2);
        }
        protected override void WriteFooter(Stream output)
        {
            BinaryWriter writer = new BinaryWriter(output);

            writer.Write(Checksum);
            writer.Write(Padding);
        }

        public Tournament Tournament
        {
            get
            {
                return new Tournament(Data);
            }
            set
            {
                Data = value.Data;
            }
        }
    }
}
